@@ -13,8 +13,9 @@ Readonly my $_360_DEGREES       => 360;
 Readonly my $FULLPAGE_OCR_SCALE => 0.8;
 my $SPACE = q{ };
 my $EMPTY = q{};
+my $device;
 
-our $VERSION = '2.2.1';
+our $VERSION = '2.3.0';
 
 use Glib::Object::Subclass GooCanvas2::Canvas::, signals => {
     'zoom-changed' => {
@@ -36,6 +37,10 @@ use Glib::Object::Subclass GooCanvas2::Canvas::, signals => {
 sub INIT_INSTANCE {
     my $self = shift;
 
+    my $display = Gtk3::Gdk::Display::get_default;
+    my $manager = $display->get_device_manager;
+    $device = $manager->get_client_pointer;
+
     # Set up the canvas
     $self->signal_connect( 'button-press-event'   => \&_button_pressed );
     $self->signal_connect( 'button-release-event' => \&_button_released );
@@ -71,6 +76,8 @@ sub INIT_INSTANCE {
     $self->{offset}{x} = 0;
     $self->{offset}{y} = 0;
 
+    # allow the widget to accessed via CSS
+    $self->set_name('gscan2pdf-ocr-output');
     return $self;
 }
 
@@ -116,9 +123,16 @@ sub add_text {
     $self->{pixbuf_size} = { width => $page->{w}, height => $page->{h} };
     $self->set_bounds( 0, 0, $page->{w}, $page->{h} );
 
+    my $style        = $self->get_style_context;
+    my $color_string = $style->get_color('normal')->to_string;
+    my $color_hex    = 'black';
+    if ( $color_string =~ /^rgb[(](\d+),(\d+),(\d+)[)]$/smx ) {
+        $color_hex = sprintf '#%02x%02x%02x', ( $1, $2, $3 );
+    }
+
     # Attach the text to the canvas
     for my $box ( @{ $page->boxes } ) {
-        boxed_text( $root, $box, [ 0, 0, 0 ], $edit_callback );
+        _boxed_text( $root, $box, [ 0, 0, 0 ], $edit_callback, $color_hex );
     }
     return;
 }
@@ -178,8 +192,8 @@ sub get_offset {
 
 # Draw text on the canvas with a box around it
 
-sub boxed_text {
-    my ( $root, $box, $transformation, $edit_callback ) = @_;
+sub _boxed_text {
+    my ( $root, $box, $transformation, $edit_callback, $text_color ) = @_;
     my ( $rotation, $x0, $y0 ) = @{$transformation};
     my ( $x1, $y1, $x2, $y2 ) = @{ $box->{bbox} };
     my $x_size = abs $x2 - $x1;
@@ -234,13 +248,14 @@ sub boxed_text {
 
         # create text and then scale, shift & rotate it into the bounding box
         my $text = GooCanvas2::CanvasText->new(
-            parent => $g,
-            text   => $box->{text},
-            x      => ( $x_size / 2 ),
-            y      => ( $y_size / 2 ),
-            width  => -1,
-            anchor => 'center',
-            'font' => 'Sans'
+            parent       => $g,
+            text         => $box->{text},
+            x            => ( $x_size / 2 ),
+            y            => ( $y_size / 2 ),
+            width        => -1,
+            anchor       => 'center',
+            'font'       => 'Sans',
+            'fill-color' => $text_color,
         );
         my $angle  = -( $textangle + $rotation ) % $_360_DEGREES;
         my $bounds = $text->get_bounds;
@@ -263,8 +278,8 @@ sub boxed_text {
     }
     if ( $box->{contents} ) {
         for my $box ( @{ $box->{contents} } ) {
-            boxed_text( $g, $box, [ $textangle + $rotation, $x1, $y1 ],
-                $edit_callback );
+            _boxed_text( $g, $box, [ $textangle + $rotation, $x1, $y1 ],
+                $edit_callback, $text_color );
         }
     }
 
@@ -488,7 +503,11 @@ sub _button_pressed {
     # left mouse button
     if ( $event->button != 1 ) { return FALSE }
 
-    $self->{drag_start} = { x => $event->x, y => $event->y };
+    # Using the root window x,y position for dragging the canvas, as the
+    # values returned by $event->x and y cause a bouncing effect, and
+    # only the value since the last event is required.
+    my ( $screen, $x, $y ) = $device->get_position;
+    $self->{drag_start} = { x => $x, y => $y };
     $self->{dragging} = TRUE;
 
     #    $self->update_cursor( $event->x, $event->y );
@@ -511,12 +530,10 @@ sub _motion {
 
     my $offset = $self->get_offset;
     my $zoom   = $self->get_scale;
-    my $offset_x =
-      $offset->{x} + ( $event->x - $self->{drag_start}{x} ) / $zoom;
-    my $offset_y =
-      $offset->{y} + ( $event->y - $self->{drag_start}{y} ) / $zoom;
-    ( $self->{drag_start}{x}, $self->{drag_start}{y} ) =
-      ( $event->x, $event->y );
+    my ( $screen, $x, $y ) = $device->get_position;
+    my $offset_x = $offset->{x} + ( $x - $self->{drag_start}{x} ) / $zoom;
+    my $offset_y = $offset->{y} + ( $y - $self->{drag_start}{y} ) / $zoom;
+    ( $self->{drag_start}{x}, $self->{drag_start}{y} ) = ( $x, $y );
     $self->set_offset( $offset_x, $offset_y );
     return;
 }
