@@ -43,10 +43,12 @@ BEGIN {
 # from http://gtk2-perl.sourceforge.net/doc/subclassing_widgets_in_perl.html
 use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
     'new-scan' => {
-        param_types => [ 'Glib::String', 'Glib::UInt' ], # filename, page number
+        param_types =>
+          [ 'Glib::String', 'Glib::UInt', 'Glib::Float', 'Glib::Float' ]
+        ,    # filename, page number, x-resolution, y-resolution
     },
     'changed-device' => {
-        param_types => ['Glib::String'],                 # device name
+        param_types => ['Glib::String'],    # device name
     },
     'changed-device-list' => {
         param_types => ['Glib::Scalar'],    # array of hashes with device info
@@ -267,7 +269,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
     ),
   ];
 
-our $VERSION = '2.2.1';
+our $VERSION = '2.3.0';
 
 my ( $d_sane, $logger );
 my $tolerance             = 1;
@@ -649,7 +651,7 @@ sub _save_profile_callback {
                         'gtk-cancel' => 'cancel'
                     );
                     $label = Gtk3::Label->new($warning);
-                    $dialog2->vbox->add($label);
+                    $dialog2->get_content_area->add($label);
                     $label->show;
                     if ( $dialog2->run eq 'ok' ) {
                         $parent->save_current_profile( $entry->get_text );
@@ -1231,6 +1233,22 @@ sub update_options {
     # so don't set this until we have finished
     $self->set( 'available-scan-options', $new_options );
 
+    # Remove buttons from $current_scan_options to prevent buttons which cause
+    # reloads from setting off infinite loops
+    my @buttons = ();
+    my $iter    = $current_scan_options->each_backend_option;
+    while ( my $i = $iter->() ) {
+        my ( $name, $val ) =
+          $current_scan_options->get_backend_option_by_index($i);
+        my $opt = $options->by_name($name);
+        if ( defined( $opt->{type} ) and $opt->{type} == SANE_TYPE_BUTTON ) {
+            push @buttons, $name;
+        }
+    }
+    for my $button (@buttons) {
+        $current_scan_options->remove_backend_option_by_name($button);
+    }
+
     # Reapply current options to ensure the same values are still set.
     $self->set_current_scan_options($current_scan_options);
 
@@ -1787,11 +1805,10 @@ sub set_options {
         title           => $d_sane->get( $opt->{title} ),
         destroy         => TRUE,
     );
-    my $vbox   = $window->vbox;
     my $canvas = Goo::Canvas->new;
     $canvas->set_size_request( $CANVAS_SIZE, $CANVAS_SIZE );
     $canvas->{border} = $CANVAS_BORDER;
-    $vbox->add($canvas);
+    $window->get_content_area->add($canvas);
     my $root = $canvas->get_root_item;
 
     $canvas->signal_connect(
@@ -1849,13 +1866,9 @@ sub set_options {
           sort { $a <=> $b } @{ $opt->{constraint} };
     }
 
-    # HBox for buttons
-    my $hbox = Gtk3::HBox->new;
-    $vbox->pack_start( $hbox, FALSE, TRUE, 0 );
-
     # Apply button
     my $abutton = Gtk3::Button->new_from_stock('gtk-apply');
-    $hbox->pack_start( $abutton, TRUE, TRUE, 0 );
+    $window->add_button($abutton);
     $abutton->signal_connect(
         clicked => sub {
             $self->set_option( $opt, $canvas->{val} );
@@ -1869,7 +1882,7 @@ sub set_options {
 
     # Cancel button
     my $cbutton = Gtk3::Button->new_from_stock('gtk-cancel');
-    $hbox->pack_end( $cbutton, FALSE, FALSE, 0 );
+    $window->add_button($cbutton);
     $cbutton->signal_connect( clicked => sub { $window->destroy } );
 
 # Have to show the window before updating it otherwise is doesn't know how big it is
@@ -2198,6 +2211,54 @@ sub make_progress_string {
     return sprintf __('Scanning page %d'), $i;
 }
 
+sub get_xy_resolution {
+    my ($self) = @_;
+    my $options = $self->get('available-scan-options');
+    if ( not defined $options ) { return }
+    my $resolution = $options->by_name(SANE_NAME_SCAN_RESOLUTION);
+    my $x          = $options->by_name('x-resolution');
+    my $y          = $options->by_name('y-resolution');
+    if ( defined $resolution ) { $resolution = $resolution->{val} }
+    if ( defined $x )          { $x          = $x->{val} }
+    if ( defined $y )          { $y          = $y->{val} }
+
+    # Potentially, a scanner could offer all three options, but then unset
+    # resolution once the other two have been set.
+    if ( defined $resolution ) {
+
+        # The resolution option, plus one of the other two, is defined.
+        # Most sensibly, we should look at the order they were set.
+        # However, if none of them are in current-scan-options, they still have
+        # their default setting, and which of those gets priority is certainly
+        # scanner specific.
+        if ( defined $x or defined $y ) {
+            my $current_scan_options = $self->get('current-scan-options');
+            my $iter = $current_scan_options->each_backend_option;
+            while ( my $i = $iter->() ) {
+                my ( $name, $val ) =
+                  $current_scan_options->get_backend_option_by_index($i);
+                if ( $name eq SANE_NAME_SCAN_RESOLUTION ) {
+                    $x = $val;
+                    $y = $val;
+                }
+                elsif ( $name eq 'x-resolution' ) {
+                    $x = $val;
+                }
+                elsif ( $name eq 'y-resolution' ) {
+                    $y = $val;
+                }
+            }
+        }
+        else {
+            return $resolution, $resolution;
+        }
+    }
+
+    if ( not defined $x ) { $x = $Gscan2pdf::Document::POINTS_PER_INCH }
+    if ( not defined $y ) { $y = $Gscan2pdf::Document::POINTS_PER_INCH }
+    return $x, $y;
+}
+
 1;
 
 __END__
